﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace VZInterfaceTestHandbook
{
  class FirmwareUpdate
  {
    IComIf comIf;
    /**
     * @param com: uses Interface IComIf
     */
    public FirmwareUpdate(IComIf com)
    {
      comIf = com;
    }

    /**
     * Start flash erase or programming
     * @return success = false
     */
    public Boolean UploadEraseFlash()
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[4];
      Byte[] bufRecv = new Byte[3];
      Boolean iReturn;

      // command code
      bufSnd[0] = 0x01;
      bufSnd[1] = 0xB0;
      bufSnd[2] = 0x01;
      bufSnd[3] = 0x00;

      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return true;
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
        {
          wp2.ErrorSummary(bufRecv[2]);
          return true;
        }
        else
        {
          return false;
        }
      }
    }

    /**
     * Start uploading data
     * @param fwDataLength: Length of FW file
     * @return success = false
     */
    public Boolean UploadDataStart(Byte[] fWDataLength)
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[7];
      Byte[] bufRecv = new Byte[3];
      Boolean iReturn;

      // command code
      bufSnd[0] = 0x01;
      bufSnd[1] = 0xB1;
      bufSnd[2] = 0x04;
      bufSnd[3] = fWDataLength[0];
      bufSnd[4] = fWDataLength[1];
      bufSnd[5] = fWDataLength[2];
      bufSnd[6] = fWDataLength[3];

      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return true;
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
        {
          wp2.ErrorSummary(bufRecv[2]);
          return true;
        }
        else
        {
          return false;
        }
      }
    }

    /**
     * Upload data block
     * @param dataLength: Length of data block
     * @param packetCounter: packet number
     * @param fWData: FW array
     * @return success = false
     */
    public Boolean UploadData(int dataLength, int packetCounter, Byte[] fWData)
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[dataLength + 4];
      Byte[] bufRecv = new Byte[3];
      Boolean iReturn;

      // command code
      bufSnd[0] = 0x05;
      bufSnd[1] = 0xB2;
      bufSnd[2] = Convert.ToByte((dataLength & 0x0000FF00) >> 8);
      bufSnd[3] = Convert.ToByte(dataLength & 0x000000FF);
      for (int i = 0; i < dataLength; i++)
      {
        bufSnd[i + 4] = fWData[i + packetCounter];
      }

      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return true;
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
        {
          wp2.ErrorSummary(bufRecv[2]);
          return true;
        }
        else
        {
          return false;
        }
      }
    }

    /**
     * End of data
     * @return success = false
     */
    public Boolean UploadStop()
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[4];
      Byte[] bufRecv = new Byte[3];
      Boolean iReturn;

      // command code
      bufSnd[0] = 0x01;
      bufSnd[1] = 0xB3;
      bufSnd[2] = 0x01;
      bufSnd[3] = 0x00;

      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return true;
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
        {
          wp2.ErrorSummary(bufRecv[2]);
          return true;
        }
        else
        {
          return false;
        }
      }
    }

    /**
     * Downgrade command
     * Only necessary on EYE12/FB4 platforms
     * @return success = false
     */
    public Boolean FWDowngrade()
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[4];
      Byte[] bufRecv = new Byte[3];
      Boolean iReturn;

      // command code
      bufSnd[0] = 0x01;
      bufSnd[1] = 0xB4;
      bufSnd[2] = 0x01;
      bufSnd[3] = 0x00;

      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return true;
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
        {
          wp2.ErrorSummary(bufRecv[2]);
          return true;
        }
        else
        {
          return false;
        }
      }
    }

    /**
     * Check progress of flash erasing or programming
     * @return progress in percent form 0 to 100, failure = 200
     */
    public int GetUploadEraseFlash()
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[3];
      Byte[] bufRecv = new Byte[4];
      Boolean iReturn;

      // command code
      bufSnd[0] = 0x00;
      bufSnd[1] = 0xB0;
      bufSnd[2] = 0x00;

      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        return 200;
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
        {
          wp2.ErrorSummary(bufRecv[2]);
          return 200;
        }
        else
          return 100 - Convert.ToInt32(bufRecv[3]);
      }
    }

    /**
     * Get the buildnumber
     * @return buildnumber, failure = null
     */
    public Byte[] GetBuildnumber()
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[3];
      Byte[] bufRecv = new Byte[17];
      Boolean iReturn;

      // command code
      bufSnd[0] = 0x00;
      bufSnd[1] = 0x1C;
      bufSnd[2] = 0x00;

      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return null;
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
        {
          wp2.ErrorSummary(bufRecv[2]);
          return null;
        }
        else
        {
          Byte[] buf = new Byte[14];
          for (int i = 0; i < buf.Length; i++)
          {
            buf[i] = bufRecv[i + 3];
          }
          return buf;
        }
      }
    }
  }
}
