﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace VZInterfaceTestHandbook
{
  class MultiUnicast
  {
    IComIf comIf;
    /**
     * @param com: uses Interface IComIf
     */
    public MultiUnicast(IComIf com)
    {
      comIf = com;
    }

    public Boolean MultiUnicastMode(Byte streamingMode)
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[4];
      Byte[] bufRecv = new Byte[3];
      Boolean iReturn;

      bufSnd[0] = 0x01;
      bufSnd[1] = 0x75;
      bufSnd[2] = 0x01;
      bufSnd[3] = streamingMode;

      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return true;
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
        {
          wp2.ErrorSummary(bufRecv[2]);
          return true;
        }
        else
          return false;
      }
    }

    public Boolean MulticastIPAddress(Byte[] multicastIP)
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[7];
      Byte[] bufRecv = new Byte[3];
      Boolean iReturn;

      bufSnd[0] = 0x01;
      bufSnd[1] = 0x77;
      bufSnd[2] = 0x04;
      bufSnd[3] = multicastIP[0];
      bufSnd[4] = multicastIP[1];
      bufSnd[5] = multicastIP[2];
      bufSnd[6] = multicastIP[3];

      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return true;
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
        {
          wp2.ErrorSummary(bufRecv[2]);
          return true;
        }
        else
          return false;
      }
    }

    public Boolean MultiUnicastPort(Byte[] streamingPort)
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[5];
      Byte[] bufRecv = new Byte[3];
      Boolean iReturn;

      bufSnd[0] = 0x01;
      bufSnd[1] = 0x78;
      bufSnd[2] = 0x02;
      bufSnd[3] = streamingPort[0];
      bufSnd[4] = streamingPort[1];

      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return true;
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
        {
          wp2.ErrorSummary(bufRecv[2]);
          return true;
        }
        else
          return false;
      }
    }

    public Boolean Start()
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[4];
      Byte[] bufRecv = new Byte[3];
      Boolean iReturn;

      bufSnd[0] = 0x01;
      bufSnd[1] = 0x5F;
      bufSnd[2] = 0x01;
      bufSnd[3] = 0x01;

      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return true;
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
        {
          wp2.ErrorSummary(bufRecv[2]);
          return true;
        }
        else
          return false;
      }
    }

    public Boolean Stop()
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[4];
      Byte[] bufRecv = new Byte[3];
      Boolean iReturn;

      bufSnd[0] = 0x01;
      bufSnd[1] = 0x5F;
      bufSnd[2] = 0x01;
      bufSnd[3] = 0x00;

      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return true;
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
        {
          wp2.ErrorSummary(bufRecv[2]);
          return true;
        }
        else
        {
          MessageBox.Show("Stopped Unicast Stream", "Information",
                         MessageBoxButtons.OK);
          return false;
        }
      }
    }

    public Byte GetMode()
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[3];
      Byte[] bufRecv = new Byte[4];
      Boolean iReturn;

      bufSnd[0] = 0x00;
      bufSnd[1] = 0x75;
      bufSnd[2] = 0x00;

      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return 0xFF;
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
        {
          wp2.ErrorSummary(bufRecv[2]);
          return 0xFF;
        }
        else
        {
          return bufRecv[3];
        }
      }
    }

    public Byte[] GetPort()
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[3];
      Byte[] bufRecv = new Byte[5];
      Boolean iReturn;

      bufSnd[0] = 0x00;
      bufSnd[1] = 0x78;
      bufSnd[2] = 0x00;

      Byte[] buf = new Byte[2];
      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return null;
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
        {
          wp2.ErrorSummary(bufRecv[2]);
          return null;
        }
        else
        {

          for (int i = 0; i < buf.Length; i++)
          {
            buf[i] = bufRecv[i + 3];
          }
          return buf;
        }
      }
    }

    public Byte[] GetMulticastIP()
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[3];
      Byte[] bufRecv = new Byte[7];
      Boolean iReturn;

      bufSnd[0] = 0x00;
      bufSnd[1] = 0x77;
      bufSnd[2] = 0x00;

      Byte[] buf = new Byte[4];
      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return null;
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
        {
          wp2.ErrorSummary(bufRecv[2]);
          return null;
        }
        else
        {
          for (int i = 0; i < buf.Length; i++)
          {
            buf[i] = bufRecv[i + 3];
          }
          return buf;
        }
      }
    }

  }
}
