﻿
using System;
using System.Collections.Generic;

namespace VZInterfaceTestHandbook
{
  public partial class RS232Com : IComIf
  {
    private System.IO.Ports.SerialPort comPort = new System.IO.Ports.SerialPort();
    int iTimeout = 1000;

    /**
     * Open the connection
     */
    public Boolean Open()
    {
      try
      {
        if (isConnected())
          comPort.Close();
        comPort.Open();
      }
      catch (Exception)
      {
      }
      return !comPort.IsOpen;
    }

    /**
     * Close the connection
     */
    public Boolean Close()
    {
      try
      {
        if (isConnected())
          comPort.Close();
      }
      catch (Exception)
      {
      }
      return false;
    }

    public InterfaceType GetInterfaceType()
    {
      return InterfaceType.RS232;
    }

    /**
     * Configures the comport name and the baudrate
     * @param strComName: comport name
     * @param baudRate: sets the baudrate
     */
    public void Configure(string strComName, int baudRate)
    {
      try
      {
        comPort.PortName = strComName;
        comPort.BaudRate = baudRate;
      }
      catch (Exception)
      {
      }
    }

    /**
     * Sets the baudrate
     * @param baudRate: sets the baudrate to 
     *                  the value of comPort.BaudRate
     */
    public Boolean SetBaudrate(int baudRate)
    {
      try
      {
        comPort.BaudRate = baudRate;
      }
      catch (Exception)
      {
      }
      return false;
    }

    /**
     * Check connection state
     * @return open = true
     */
    public Boolean isConnected()
    {
      return comPort.IsOpen;
    }

    /**
     * Write bytes
     * @param buffer: bytes to write
     * @param count: number of bytes
     * @return: success = false
     */
    public Boolean Write(Byte[] buffer, int count)
    {
      if (!isConnected())
        return true;
      try
      {
        comPort.Write(buffer, 0, count);
      }
      catch (Exception)
      {
        return true;
      }
      return false;
    }

    /**
     * Set the timeout
     * @param int timeout: time in ms
     */
    public Boolean ReadTimeout(int timeout)
    {
      iTimeout = timeout;
      return false;
    }

    /**
     * @return available bytes for read
     */
    public int BytesToRead()
    {
      int iToRead = 0;

      if (!isConnected())
        return 0;
      try
      {
        iToRead = comPort.BytesToRead;
      }
      catch (Exception)
      {
      }
      return iToRead;
    }

    /**
     * Clear queue 
     */
    public void Flush()
    {
      if (!isConnected())
        return;
      try
      {
        comPort.DiscardInBuffer();
      }
      catch (Exception)
      {
      }
    }

    /**
     * Read bytes currently available
     * @param buffer: to be filled
     * @param count: number of bytes to be read
     * returns read bytes
     */
    public int Read(out Byte[] buffer, int count)
    {
      List<Byte> lRecvBuffer = new List<Byte>();

      if (!isConnected())
      {
        buffer = null;
        return -1;
      }

      try
      {
        comPort.ReadTimeout = iTimeout;
        while (count-- > 0)
          lRecvBuffer.Add((Byte)comPort.ReadByte());
      }
      catch
      {
      }

      buffer = lRecvBuffer.ToArray();
      return lRecvBuffer.Count;
    }

    /**
     * Read one byte
     * @param byteRecv: to be filled
     */
    public Boolean ReadByte(ref Byte byteRecv)
    {
      if (!isConnected())
        return true;

      try
      {
        comPort.ReadTimeout = iTimeout;
        byteRecv = (Byte)comPort.ReadByte();
      }
      catch
      {
        return true;
      }

      return false;
    }

  }
}
