
using System;
using System.Windows;
using System.Windows.Forms;
using System.Collections;
using System.Collections.Generic;



namespace VZInterfaceTestHandbook
{
  public partial class Wolfprot2
  {

    /**
     * constants
     */
    public const Byte HEADER_DIR = 0x01;          //0=GET, 1=SET
    public const Byte HEADER_DIR_GET = 0x00;      //GET
    public const Byte HEADER_DIR_SET = 0x01;      //SET
    public const Byte HEADER_EXT_HDR = 0x02;      //0=1 Byte Header, 1=2 Byte Header
    public const Byte HEADER_EXT_LEN = 0x04;      //0=1 Byte Length, 1=2 Byte Length
    public const Byte HEADER_EXT_CMD = 0x08;      //0=1 Byte Command, 1=2 Byte Command
    public const Byte HEADER_PROT2 = 0xF0;        //Prot2 mask
    public const Byte HEADER_ERR = 0x80;          //0=OK,  1=ERR

    public const Byte EXTHEADER_EXT_LEN = 0x01;   //32bit length

    public const Byte PROT2_GET = 0x00;
    public const Byte PROT2_SET = 0x01;
    public const Byte PROT2 = 0x00;

    public const Byte ERR_TIMEOUT = 0x01;
    public const Byte ERR_UNKNOWN_CMD = 0x02;
    public const Byte ERR_UNKNOWN_PAR = 0x03;
    public const Byte ERR_INVALID_LEN = 0x04;
    public const Byte ERR_FIFO_FULL = 0x05;
    public const Byte ERR_FW_UPDATE = 0x06;
    public const Byte ERR_ACCESS_DENIED = 0x07;
    public const Byte ERR_AUTH_REQUIRED = 0x08;
    public const Byte ERR_BUSY = 0x09;

    private const Int32 MAX_MESG_SIZE = 2048;

    private List<Byte> lRecvBuffer = new List<Byte>();
    private UInt32 uiMinBytes, uiSize;

    IComIf comIf;

    public Wolfprot2(IComIf com)
    {
      comIf = com;
    }

    public void ResetParser()
    {
      uiMinBytes = 0;
      uiSize = 0;
      lRecvBuffer.Clear();
    }

    /**
     * @return true if complete frame was received
     */
    public Boolean ParseData(Byte ucData, IComIf comIf)
    {

      lRecvBuffer.Add(ucData);

      if (lRecvBuffer.Count < 3)
        return false;

      if (uiSize == 0)
      {
        // Calculate minimum required frame length
        uiMinBytes = 3;
        if ((lRecvBuffer.ToArray()[0] & HEADER_ERR) == 0)
        {
          if ((lRecvBuffer.ToArray()[0] & HEADER_EXT_HDR) != 0)
            uiMinBytes++;
          if (((lRecvBuffer.ToArray()[0] & HEADER_EXT_HDR) != 0) && ((lRecvBuffer.ToArray()[1] & EXTHEADER_EXT_LEN) != 0))
            uiMinBytes += 3;
          else
            if ((lRecvBuffer.ToArray()[0] & HEADER_EXT_LEN) != 0)
              uiMinBytes++;
          if ((lRecvBuffer.ToArray()[0] & HEADER_EXT_CMD) != 0)
            uiMinBytes++;
        }

        if (lRecvBuffer.Count < uiMinBytes)
          return false;

        // Extract and calculate length information
        if ((lRecvBuffer.ToArray()[0] & HEADER_ERR) != 0)
        {
          uiSize = 0;
        }
        else if (((lRecvBuffer.ToArray()[0] & HEADER_EXT_HDR) != 0) && ((lRecvBuffer.ToArray()[1] & EXTHEADER_EXT_LEN) != 0))
        {
          uiSize = (((UInt32)lRecvBuffer.ToArray()[uiMinBytes - 4]) << 24) |
                   (((UInt32)lRecvBuffer.ToArray()[uiMinBytes - 3]) << 16) |
                   (((UInt32)lRecvBuffer.ToArray()[uiMinBytes - 2]) << 8) |
                    ((UInt32)lRecvBuffer.ToArray()[uiMinBytes - 1]);
        }
        else if ((lRecvBuffer.ToArray()[0] & HEADER_EXT_LEN) != 0)
        {
          uiSize = (((UInt32)lRecvBuffer.ToArray()[uiMinBytes - 2]) << 8) |
                    ((UInt32)lRecvBuffer.ToArray()[uiMinBytes - 1]);
        }
        else
        {
          uiSize = (UInt32)lRecvBuffer.ToArray()[uiMinBytes - 1];
        }
      }

      // Reset parser if frame complete
      
      if (comIf.GetInterfaceType().Equals(InterfaceType.USB) && USBManager.currUSBDevice.Tag.ToString().Contains("EYE-12"))  //special EYE USB handling (16bit)
      {
        if (uiSize == 0 && lRecvBuffer.Count > 3)
        {
          lRecvBuffer.RemoveAt(lRecvBuffer.Count - 1);
          return true;
        }
        else
        {
          if (lRecvBuffer.Count >= (uiSize + uiMinBytes))
          {
            //EYE Devices always return even numbers -> cut last byte
            if ((uiSize + uiMinBytes) % 2 == 1)
            {
              if (lRecvBuffer.Count >= (uiSize + uiMinBytes + 1))
              {
                lRecvBuffer.RemoveAt(lRecvBuffer.Count - 1);
                return true;
              }
            }
            else
              return true;
          }
          return false;
        }
      }
      else
      {
        if (lRecvBuffer.Count >= (uiSize + uiMinBytes))
        {
          return true;
        }
        return false;
      }
      return false;
    }

    public Byte[] GetData()
    {
      return lRecvBuffer.ToArray();
    }


    /**
     * Execute a wolfprot command.
     * Receiving commands is done byte for byte using a parser.
     * Use this for short commands.
     * @param baCommand: Wolfprot command request
     * @param baReceived: Wolfprot command reply
     * @return success = false
     */
    //@@@START_EXECCMD@@@
    public Boolean execCmd(Byte[] baCommand, ref Byte[] baReceived)
    {
      Boolean bError = false;

      ResetParser();
      bError |= comIf.ReadTimeout(0);

      // Remove all bytes from queue
      comIf.Flush();

      bError |= comIf.Write(baCommand, baCommand.Length);
      bError |= comIf.ReadTimeout(3000);  //3 seconds

      Byte byRecv = 0;
      do
      {
        bError |= comIf.ReadByte(ref byRecv);
      } while (!ParseData(byRecv, comIf) && !bError);

      baReceived = GetData();

      return bError;
    }
    //@@@STOP_EXECCMD@@@


    /**
     * Execute a wolfprot command, fast version.
     * Receiving commands is done in blocks without parser.
     * Use this for long commands, like picture transfer.
     * @param baCommand: Wolfprot command request
     * @param baReceived: Wolfprot command reply
     * @param blocklength: Number of bytes to receive
     * @return success = false
     */
    //@@@START_EXECCMDFAST@@@
    public Boolean execCmdFast(Byte[] baCommand, ref Byte[] baReceived, int blocklength)
    {
      Boolean bError = false;
      Byte[] bufRec = new Byte[blocklength];

      int cnt = blocklength;
      bError |= comIf.ReadTimeout(0);

      // Remove all bytes from queue
      comIf.Flush();

      bError |= comIf.Write(baCommand, baCommand.Length);
      bError |= comIf.ReadTimeout(10000);

      int check = 0;  // Length of chunck read

      int ptr = 0;
      do // Read loop
      {
        check = comIf.Read(out bufRec, cnt);
        cnt = cnt - check;

        Array.Copy(bufRec, 0, baReceived, ptr, check);
        //Check the header
        if((ptr == 0) && (check <= 3))     // Wolfprot error message (1 byte header + 1 byte cmd)
        {
          if (check == 0)                  // No bytes received, device switched off
          {
            baReceived = null;
            bError |= true;
            break;
          }
          if ((baReceived[0] & 0x80) == 0x80)
            break;
        }

        ptr += check;
        if (check != -1)
          bError |= false;

      } while ((cnt>0) && !bError);

      return bError;
    }
    //@@@STOP_EXECCMDFAST@@@


    /** 
     * Shows success or error message
     */
    //@@@START_ERRORSUMMARY@@@
    public void ErrorSummary(Byte ByteError)
    {
      switch (ByteError)
      {
        case 0x01:
          MessageBox.Show("Timeout");
          break;
        case 0x02:
          MessageBox.Show("Unknown command");
          break;
        case 0x03:
          MessageBox.Show("Unknown parameter");
          break;
        case 0x04:
          MessageBox.Show("Invalid length");
          break;
        case 0x05:
          MessageBox.Show("Fifo full");
          break;
        case 0x06:
          MessageBox.Show("Firmware update");
          break;
        case 0x07:
          MessageBox.Show("Access denied");
          break;
        case 0x08:
          MessageBox.Show("Auth required");
          break;
        case 0x09:
          MessageBox.Show("Busy");
          break;
        default:
          MessageBox.Show("Unknown error");
          break;
      }
    }
    //@@@STOP_ERRORSUMMARY@@@//
  }
}
