﻿
using System;
using System.Windows;
using System.Windows.Forms;
using System.Collections;
using System.Collections.Generic;


namespace VZInterfaceTestHandbook
{
  public partial class ZoomSettings
  {
    //@@@START_ZOOMSETTINGS@@@
    const int STEP = (0x2000 / 100) + 1;  // (Digital Zoom Range + Analog Zoom Range)/Scale of Slider
    const int STEP2 = (0x1000 / 100) + 1; // (Combined Digital and Analog Zoom Range)/Scale of Slider
    IComIf comIf;

    /**
     * Zoom commands
     * @param com: uses Interface IComIf
     */
    public ZoomSettings(IComIf com)
    {
      comIf = com;
    }
    //@@@STOP_ZOOMSETTINGS@@@

    //@@@START_STARTZOOMWIDE@@@
    public void StartZoomWide()
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      /*Declaration*/
      Byte[] bufSnd = new Byte[4];
      Byte[] bufRecv = new Byte[3];
      Boolean iReturn;

      /*Assignment*/
      bufSnd[0] = 0x01;
      bufSnd[1] = 0x20;
      bufSnd[2] = 0x01;
      bufSnd[3] = 0x11;

      /*Send command*/
      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
          wp2.ErrorSummary(bufRecv[2]);
      }
    }
    //@@@STOP_STARTZOOMWIDE@@@

    public void StartZoomTele()
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[4];
      Byte[] bufRecv = new Byte[3];
      Boolean iReturn;

      bufSnd[0] = 0x01;
      bufSnd[1] = 0x20;
      bufSnd[2] = 0x01;
      bufSnd[3] = 0x12;

      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
          wp2.ErrorSummary(bufRecv[2]);
      }
    }

    public void StopZoom()
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      /*Declaration*/
      Byte[] bufSnd = new Byte[4];
      Byte[] bufRecv = new Byte[3];
      Boolean iReturn;

      /*Assignment*/
      bufSnd[0] = 0x01;
      bufSnd[1] = 0x2F;
      bufSnd[2] = 0x01;
      bufSnd[3] = 0x00;

      /*Send command*/
      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
          wp2.ErrorSummary(bufRecv[2]);
      }
    }


    //@@@START_ZOOMSLIDERSPEED@@@
    public void ZoomSliderSpeed(TrackBar trBar)
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      /*Declaration*/
      Byte[] bufSnd = new Byte[6];
      Byte[] bufRecv = new Byte[40];
      Boolean iReturn;

      /*Assignment*/
      bufSnd[0] = 0x01;
      bufSnd[1] = 0x20;
      bufSnd[2] = 0x03;

      // Zoom wide
      if (trBar.Value < 0)
      {
        bufSnd[3] = 0x01;
        bufSnd[4] = 0x00;
        bufSnd[5] = Convert.ToByte(trBar.Value * -1);
      }
      // Zoom tele
      if (trBar.Value > 0)
      {
        bufSnd[3] = 0x02;
        bufSnd[4] = 0x00;
        bufSnd[5] = Convert.ToByte(trBar.Value);
      }

      /*Send command*/
      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
          wp2.ErrorSummary(bufRecv[2]);
      }
    }
    //@@@STOP_ZOOMSLIDERSPEED@@@

    //@@@START_ZOOMSLIDERSETPOS@@@
    public void ZoomSliderSetPos(TrackBar trBar)
    {
      Boolean bDigitalZoomSupported = false;

      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Boolean iReturn;
      /*Declaration*/
      Byte[] bufSndAnalog = new Byte[5];
      Byte[] bufRecvAnalog = new Byte[3];

      Byte[] bufSndDigital = new Byte[5];
      Byte[] bufRecvDigital = new Byte[3];

      /*Assignment*/
      // Analog
      bufSndAnalog[0] = 0x01;
      bufSndAnalog[1] = 0x20;
      bufSndAnalog[2] = 0x02;

      // Digital
      bufSndDigital[0] = 0x01;
      bufSndDigital[1] = 0x28;
      bufSndDigital[2] = 0x02;

      // Check if digital zoom is supported
      iReturn = wp2.execCmd(bufSndDigital, ref bufRecvDigital);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
      }
      else
      {
        if ((bufRecvDigital[0] & 0x80) == 0x80)
        {
          if (bufRecvDigital[2] == Wolfprot2.ERR_UNKNOWN_CMD)
            bDigitalZoomSupported = false;
          else
            wp2.ErrorSummary(bufRecvDigital[2]);
        }
        else
        {
          bDigitalZoomSupported = true;
        }
      }
      if (bDigitalZoomSupported)
      {
        if (trBar.Value > 50)
        {
          // In digital zoom range
          // i: number of bytes the digitalzoom uses for position
          int i = (trBar.Value - 50) * STEP;

          // Convert a long byte in two short bytes, which the program is able to read
          bufSndDigital[3] = Convert.ToByte((i & 0x0000FF00) >> 8);
          bufSndDigital[4] = Convert.ToByte(i & 0x000000FF);

          /*Send command*/
          iReturn = wp2.execCmd(bufSndDigital, ref bufRecvDigital);
          if (iReturn == true)
          {
            MessageBox.Show("No communication!", "Error",
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error);
          }
          else
          {
            if ((bufRecvDigital[0] & 0x80) == 0x80)
              wp2.ErrorSummary(bufRecvDigital[2]);
          }
        }
        else
        {
          // In analog zoom range
          // i: number of bytes the digitalzoom uses for position
          int i = trBar.Value * STEP;

          // Convert a long byte in two short bytes, which the program is able to read
          bufSndAnalog[3] = Convert.ToByte((i & 0x0000FF00) >> 8);
          bufSndAnalog[4] = Convert.ToByte(i & 0x000000FF);

          /*Send command*/
          iReturn = wp2.execCmd(bufSndAnalog, ref bufRecvAnalog);
          if (iReturn == true)
          {
            MessageBox.Show("No communication!", "Error",
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error);
          }
          else
          {
            if ((bufRecvAnalog[0] & 0x80) == 0x80)
              wp2.ErrorSummary(bufRecvAnalog[2]);
          }
        }
      }
      else
      {
        // In analog and digital zoom range
        // i: number of bytes the digitalzoom uses for position
        int i = trBar.Value * STEP2;

        // Convert a long byte in two short bytes, which the program is able to read
        bufSndAnalog[3] = Convert.ToByte((i & 0x0000FF00) >> 8);
        bufSndAnalog[4] = Convert.ToByte(i & 0x000000FF);

        /*Send command*/
        iReturn = wp2.execCmd(bufSndAnalog, ref bufRecvAnalog);
        if (iReturn == true)
        {
          MessageBox.Show("No communication!", "Error",
                          MessageBoxButtons.OK,
                          MessageBoxIcon.Error);
        }
        else
        {
          if ((bufRecvAnalog[0] & 0x80) == 0x80)
            wp2.ErrorSummary(bufRecvAnalog[2]);
        }
      }
    }
    //@@@STOP_ZOOMSLIDERSETPOS@@@

    //@@@START_ZOOMSLIDERGETPOS@@@
    public int ZoomSliderGetPos(TrackBar trBar)
    {
      Boolean bDigitalZoomSupported = false;

      Wolfprot2 wp2 = new Wolfprot2(comIf);

      /*Declaration*/
      Byte[] bufSndAnalog = new Byte[3];
      Byte[] bufRecvAnalog = new Byte[4];

      Byte[] bufSndDigital = new Byte[3];
      Byte[] bufRecvDigital = new Byte[4];

      Boolean iReturnAnalog;
      Boolean iReturnDigital;

      /*Assignment*/
      bufSndAnalog[0] = 0x00;
      bufSndAnalog[1] = 0x20;
      bufSndAnalog[2] = 0x00;

      bufSndDigital[0] = 0x00;
      bufSndDigital[1] = 0x28;
      bufSndDigital[2] = 0x00;

      /*Send command*/
      iReturnAnalog = wp2.execCmd(bufSndAnalog, ref bufRecvAnalog);
      iReturnDigital = wp2.execCmd(bufSndDigital, ref bufRecvDigital);
      if (iReturnAnalog == true || iReturnDigital == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return 0;
      }
      if ((bufRecvAnalog[0] & 0x80) == 0x80)
      {
        wp2.ErrorSummary(bufRecvAnalog[2]);
        return 0;
      }
      // Not all visualizers support digital zoom
      if ((bufRecvDigital[0] & 0x80) == 0x80)
      {
        if (bufRecvDigital[2] == Wolfprot2.ERR_UNKNOWN_CMD)
          bDigitalZoomSupported = false;
        else
        {
          wp2.ErrorSummary(bufRecvDigital[2]);
          return 0;
        }
      }
      else
      {
        bDigitalZoomSupported = true;
      }
      if (bDigitalZoomSupported)
      {
        trBar.Value = (bufRecvAnalog[3] * 256
                       + bufRecvAnalog[4]
                       + bufRecvDigital[3] * 256
                       + bufRecvDigital[4]) / STEP;
      }
      else
      {
        trBar.Value = (bufRecvAnalog[3] * 256
                       + bufRecvAnalog[4]) / STEP2;
      }

      return trBar.Value;
    }
    //@@@STOP_ZOOMSLIDERGETPOS@@@
  }
}
